require 'spec_helper'


describe FuelPricesEurope::FuelPrices do


  before :each do
    VCR.eject_cassette
    VCR.insert_cassette('fuelprices', allow_playback_repeats: true)
  end

  after :each do
    FuelPricesEurope.reset
  end

  describe 'cache testing' do
    before {
      FuelPricesEurope.configure do |config|
        config.cache = ActiveSupport::Cache::NullStore.new
        config.cache_key = 'fuel_prices_europe_testing_key'
      end
    }

    context 'check if cache exists' do
      subject(:cache) { FuelPricesEurope.configuration.cache }

      it 'must not be nill' do
        expect(cache).to be_a ActiveSupport::Cache::Store
        expect(cache).to_not be nil
      end
    end

    context 'check if functionality works' do
      before { @fuel_prices = described_class.new }
      subject(:fuel_price) { @fuel_prices.find_by_country_name('Belgium') }

      it 'should find data for Belgium' do
        expect(fuel_price.country.alpha2).to eq 'BE'
      end
    end


  end


  describe 'find fuel prices by an array of countries' do
    before :each do
      @fuel_prices = described_class.new
    end

    context 'use alpha2 codes' do
      subject(:countries) { @fuel_prices.find_by_countries(['NL', 'GB', 'AT']) }

      it 'should find data for each country' do
        expect(countries['NL'].country.alpha2).to eq 'NL'
        expect(countries['GB'].country.alpha2).to eq 'GB'
        expect(countries['AT'].country.alpha2).to eq 'AT'
        expect(countries['AT'].gasoline_price).to eq 1.17
        expect(countries.size).to eq 3
      end
    end

    context 'use country names' do
      subject(:countries) { @fuel_prices.find_by_countries(['Belgium', 'France'], :name) }

      it 'should find data for each country' do
        expect(countries['Belgium'].country.alpha2).to eq 'BE'
        expect(countries['France'].country.alpha2).to eq 'FR'
        expect(countries['France'].gasoline_price).to eq 1.269
        expect(countries.size).to eq 2
      end
    end

    context 'use alpha3 codes' do
      subject(:countries) { @fuel_prices.find_by_countries(['BLR', 'HRV'], :alpha3) }

      it 'should find data for each country' do
        expect(countries['BLR'].country.name).to eq 'Belarus'
        expect(countries['HRV'].country.name).to eq 'Croatia'
        expect(countries['HRV'].gasoline_price).to eq 1.105
        expect(countries.size).to eq 2
      end
    end

  end


  context 'find fuel prices for The Netherlands' do
    before { @fuel_prices = described_class.new }
    subject(:fuel_price) { @fuel_prices.find_by_country_code('NL') }

    it 'returns the correct parsed data' do
      expect(fuel_price.country.alpha2).to eq 'NL'
      expect(fuel_price.gasoline_price).to eq 1.579
      expect(fuel_price.unleaded_95_ron_price).to eq 1.579
      expect(fuel_price.unleaded_95_ron_price_local).to eq nil
      expect(fuel_price.lpg_price).to eq 0.769
      expect(fuel_price.lpg_price_local).to eq nil
      expect(fuel_price.diesel_price).to eq 1.297
      expect(fuel_price.diesel_price_local).to eq nil
      expect(fuel_price.lpg_stations).to eq 1945
      expect(fuel_price.lpg_nozzle).to eq 'bayonet'
      expect(fuel_price.record_date).to eq Date.new(2015, 2, 6).to_datetime
    end

  end

  context 'find fuel prices for Lebanon' do
    before { @fuel_prices = described_class.new }
    subject(:fuel_price) { @fuel_prices.find_by_country_alpha3('LBN') }

    it 'returns the correct parsed data' do
      expect(fuel_price.country.alpha2).to eq 'LB'
      expect(fuel_price.gasoline_price).to eq 1.266
      expect(fuel_price.unleaded_95_ron_price).to eq 1.266
      expect(fuel_price.unleaded_95_ron_price_local).to eq ({:currency => 'LBP', :value => 2110.0})
      expect(fuel_price.lpg_price).to eq 0.702
      expect(fuel_price.lpg_price_local).to eq ({:currency => 'LBP', :value => 1170.0})
      expect(fuel_price.diesel_price).to eq 0.846
      expect(fuel_price.diesel_price_local).to eq ({:currency => 'LBP', :value => 1410.0})

      expect(fuel_price.lpg_stations).to eq nil
      expect(fuel_price.lpg_nozzle).to eq nil
      expect(fuel_price.record_date).to eq Date.new(2015, 2, 6).to_datetime
    end
  end

  context 'find fuel prices for an invalid country' do
    before { @fuel_prices = described_class.new }
    subject(:fuel_price) { @fuel_prices.find_by_country_alpha2('US') }

    it 'return nil' do
      expect(fuel_price).to be nil
    end
  end

  context 'find fuel prices for Saudi Arabia' do
    before { VCR.eject_cassette }
    before { VCR.insert_cassette('fuelprices', allow_playback_repeats: true) }
    before { @fuel_prices = described_class.new }
    subject(:fuel_price) { @fuel_prices.find_by_country_name('Saudi Arabia') }

    it 'return nil' do
      expect(fuel_price.lpg_price).to be 0.0
    end
  end


end
